;+
; NAME:
;        TVFRAME
; PURPOSE:
;        Display an image with axis.
; CATEGORY:
;        Image dispaly
; CALLING SEQUENCE:
;        TVBOX,data
; INPUTS:
;        data = Two dimensional array of numerical type
; KEYWORD PARAMETERS:
;     XRANGE   : Array with at least 2 elements giving the range for
;                the annotation of the x-axis.
;                Only min(XRANGE) and max(XRANGE) are used.
;                Default is annotation with pixel indices.
;                (should not be confused with the standard plotting keword)
;     YRANGE   : Same as XRANGE but for y-axis.
;     POSITION : (output) position parameter for the axis in device units,
;                may be used in subsequent calls to PLOT or CONTOUR.
;                Example : TVBOX,a,/aspect,position=p
;                          CONTOUR,a,nlev=10,xsty=5,ysty=5,pos=p,/dev,/noeras
;                          (no keyword like /ASPECT exists for CONTOUR)
;     /sample  : If set, nearest neighbourhood method is used in
;                resizing the data array.
;                Default : averaging or linear interpolation.
;                (Corresponds to the SAMPLE-keyword of the REBIN function)
;     /center  : If set, tickmarks are placed at the center of each pixel.
;                ( example: tvframe,randomu(seed,20,20),/sam [,/center] )
;     /aspect  : If set, the aspect ratio of the data array is preserved
;                (quadratic pixel shapes).
;     /noscale : If set, data is not scaled. (much like TV compared to TVSCL)
;     /bar     : If set, an intensity bar is displayed.
;     BRANGE   : Range for annotation of the intensity bar.
;                Default is the range of data.
;     BTITLE   : Title of the intensity bar.
;     BTICKS,BMINOR : Control the number of tickmarks for the intensity bar.
; Standard plotting keywords :
;      TITLE,SUBTITLE,XTITLE,YTITLE,TICKLEN,CHARSIZE,XSTYLE,YSTYLE,
;      XTICKS,YTICKS,XMINOR,YMINOR
;                Normally they are just passed through,
;                with the following exceptions :
;                the default value for TICKLEN is -.01 instead of .02 .
;                bit 0 of X/Y-STYLE is allways set (exact axis range).
;                bit 1 of X/Y-STYLE is  never  set (no extended axis range).
; OUTPUTS:
;        POSITION of the plot frame.
; COMMON BLOCKS:
;        none.
; SIDE EFFECTS:
;        none.
; RESTRICTIONS:
;        none.
; PROCEDURE:
;        The image is scaled to the size of the axis-box via the
;        RESIZE-function, and then displayed using TV or TVSCL.
; MODIFICATION HISTORY:
;        Written, A. Welz, Univ. Wuerzburg, Germany, Feb. 1991
;        Extended (/bar,xticks,..) by A.W. June 1992
;-
;
pro tvframemod , a    $
    , sample=sample , center=center , aspect=aspect , noscale=noscale    $
    , POSITION=POSITION    $
    , XRANGE=XRANGI , YRANGE=YRANGI   $
    , TITLE=TITLE , XTITLE=XTITLE , YTITLE=YTITLE , SUBTITLE=SUBTITLE   $
    , TICKLEN=TICKLEN , CHARSIZE=CHARSIZE   $
    , XTICKS=XTICKS , YTICKS=YTICKS , XMINOR=XMINOR , YMINOR=YMINOR $
    , XSTYLE=XSTYLI , YSTYLE=YSTYLI   $
    , bar=bar , BTITLE=BTITLE , BRANGE=BRANGE  $
    , BTICKS=BTICKS , BMINOR=BMINOR   $
    , YTICKNAME=YTICKNAME , XTICKNAME=XTICKNAME
on_error,2
sa=size(reform(a)) & if sa(0) ne 2 then goto,errout
mina=min(a,max=maxa)
;
; set keyword parameters to default values if not present
if n_elements(XRANGI) eq 0 then XRANGI=[0,sa(1)-1]
if n_elements(YRANGI) eq 0 then YRANGI=[0,sa(2)-1]
if n_elements(   TITLE) eq 0 then    TITLE=''
if n_elements(SUBTITLE) eq 0 then SUBTITLE=''
if n_elements(  XTITLE) eq 0 then   XTITLE=''
if n_elements(  YTITLE) eq 0 then   YTITLE=''
if n_elements(TICKLEN) eq 0 then TICKLEN=-.01
if n_elements(XTICKS) eq 0 then XTICKS=0
if n_elements(XMINOR) eq 0 then XMINOR=0
if n_elements(YTICKS) eq 0 then YTICKS=0
if n_elements(YMINOR) eq 0 then YMINOR=0
if n_elements(CHARSIZE) eq 0 then CHARSIZE=1.0
XSTYLE=1  &  YSTYLE=1
if n_elements(XSTYLI) eq 1 then XSTYLE=( 1 or XSTYLI ) and 29
if n_elements(YSTYLI) eq 1 then YSTYLE=( 1 or YSTYLI ) and 29
if n_elements(BTITLE) eq 0 then BTITLE=''
if n_elements(BRANGE) eq 0 then BRANGE=float([mina,maxa])
if n_elements(BTICKS) eq 0 then BTICKS=0
if n_elements(BMINOR) eq 0 then BMINOR=0
if n_elements(YTICKNAME) eq 0 then YTICKNAME='Nothing'
if n_elements(XTICKNAME) eq 0 then XTICKNAME='Nothing'
;
XRANGE=float(minmax(XRANGI))
YRANGE=float(minmax(YRANGI))
;
if keyword_set(center) then begin
    xunit=0.5*(XRANGE(1)-XRANGE(0))/float(sa(1)-1)
    yunit=0.5*(YRANGE(1)-YRANGE(0))/float(sa(2)-1)
    XRANGE(0)=XRANGE(0)-xunit  &  XRANGE(1)=XRANGE(1)+xunit
    YRANGE(0)=YRANGE(0)-yunit  &  YRANGE(1)=YRANGE(1)+yunit
endif else begin
    xunit=(XRANGE(1)-XRANGE(0))/float(sa(1)-1)
    yunit=(YRANGE(1)-YRANGE(0))/float(sa(2)-1)
    XRANGE(1)=XRANGE(1)+xunit
    YRANGE(1)=YRANGE(1)+yunit
endelse
;
plot,XRANGE,YRANGE,/nodata,xstyle=xstyle or 4,ystyle=ystyle or 4    $
     ,  TITLE=' ',XTITLE=XTITLE,YTITLE=YTITLE    $
     ,  SUBTITLE=SUBTITLE      $
     ,  TICKLEN=TICKLEN,CHARSIZE=CHARSIZE   $
     ,  color=!p.background
;
px = !x.window * !d.x_vsize     ;Position of frame in device units
py = !y.window * !d.y_vsize
sx = px(1)-px(0)                ;Size of frame in device units
sy = py(1)-py(0)
if keyword_set(bar) then sx = sx/1.25
if keyword_set(aspect) then begin
     f = float(sa(1))/sa(2)*sy/sx
     if f ge 1. then sy=sy/f else sx=sx*f
     sx=fix(sx)
endif
POSITION = [px(0),py(0),px(0)+sx,py(0)+sy]
if keyword_set(bar) then begin
   bx    = fix(px(0)+sx*1.04)
   by    = fix(py(0))
   bsx   = fix(sx*0.08)
   bsy   = fix(sy)
   barpos= [bx,by,bx+bsx,by+bsy]
endif
;
mcol=( !D.N_COLORS - 1) > 0
;
if (!d.flags and 1) ne 0 then begin

      ;  scalable pixels

     if keyword_set(sample) then b=a else b=resize(a,256,256)
     if keyword_set(noscale) then begin
         tv, 0>b<mcol ,px(0),py(0),xsize=sx,ysize=sy,/device
     endif else begin
         tvscl, b ,px(0),py(0),xsize=sx,ysize=sy,/device
     endelse
     if keyword_set(bar) then begin
        barim=findgen(1,256)/255*(maxa-mina)+mina
        if keyword_set(noscale) then begin
            tv, 0>barim<mcol ,bx,by,xsize=bsx,ysize=bsy,/device
        endif else begin
            tvscl, barim ,bx,by,xsize=bsx,ysize=bsy,/device
        endelse
     endif

endif else begin

      ;  not scalable pixels

     if sx*sy gt 10e6 then begin
        print,' do you really want to allocate ',sx*sy   $
             ,' words of memory ? [y/n]'
        answer='n'   &  read,answer
        if answer ne 'y' then return
     endif
     if keyword_set(sample) then b=resize(a,sx,sy,/sample)  $
        else b=resize(a,sx,sy)
     if keyword_set(noscale) then begin
        tv, 0>b<mcol ,px(0),py(0),/device
     endif else begin
        tvscl, b ,px(0),py(0),/device
     endelse
     if keyword_set(bar) then begin
        barim=findgen(1,bsy)/(bsy-1)*(maxa-mina)+mina
        barim=rebin(barim,bsx,bsy,/sample)
        if keyword_set(noscale) then begin
           tv, 0>barim<mcol ,bx,by,/device
        endif else begin
           tvscl, barim ,bx,by,/device
        endelse
     endif

endelse
;
if keyword_set(bar) then begin
   plot,[0,1],BRANGE,/nodata,/noerase,pos=barpos,/device,xsty=5,ysty=5
   plots,[bx+bsx,bx,bx,bx+bsx],[by,by,by+bsy,by+bsy],/device
   axis,yaxis=1,bx+bsx,/device,yrange=BRANGE,ystyle=1   $
     ,  ytitle=BTITLE $
     ,  TICKLEN=-.15,CHARSIZE=CHARSIZE,YTICKS=BTICKS,YMINOR=BMINOR
endif
;

if (ytickname(0) eq 'Nothing') then begin
if (xtickname(0) eq 'Nothing') then begin
plot,XRANGE,YRANGE,/nodata,/noerase,xstyle=XSTYLE,ystyle=YSTYLE    $
     ,  POSITION=POSITION  ,/device    $
     ,  TITLE=TITLE,XTITLE=XTITLE,YTITLE=YTITLE    $
     ,  SUBTITLE=SUBTITLE      $
     ,  TICKLEN=TICKLEN,CHARSIZE=CHARSIZE   $
     ,  XTICKS=XTICKS,XMINOR=XMINOR , YTICKS=YTICKS,YMINOR=YMINOR
endif else begin
plot,XRANGE,YRANGE,/nodata,/noerase,xstyle=XSTYLE,ystyle=YSTYLE    $
     ,  POSITION=POSITION  ,/device    $
     ,  TITLE=TITLE,XTITLE=XTITLE,YTITLE=YTITLE    $
     ,  SUBTITLE=SUBTITLE      $
     ,  TICKLEN=TICKLEN,CHARSIZE=CHARSIZE   $
     ,  XTICKS=XTICKS,XMINOR=XMINOR , YTICKS=YTICKS,YMINOR=YMINOR  $
     ,  XTICKNAME=XTICKNAME
endelse


endif else begin

plot,XRANGE,YRANGE,/nodata,/noerase,xstyle=XSTYLE,ystyle=YSTYLE    $
     ,  POSITION=POSITION  ,/device    $
     ,  TITLE=TITLE,XTITLE=XTITLE,YTITLE=YTITLE    $
     ,  SUBTITLE=SUBTITLE      $
     ,  TICKLEN=TICKLEN,CHARSIZE=CHARSIZE   $
     ,  XTICKS=XTICKS,XMINOR=XMINOR , YTICKS=YTICKS,YMINOR=YMINOR  $
     ,  YTICKNAME=YTICKNAME , XTICKNAME=XTICKNAME
endelse
;
return
;
errout: print,' TVFRAME : data must be 2-dimensional !'
return
end


; $Id: congrid.pro,v 1.5 1997/01/15 03:11:50 ali Exp $
;
; Copyright (c) 1988-1997, Research Systems, Inc.  All rights reserved.
;	Unauthorized reproduction prohibited.
;
;+
; NAME:
;	CONGRID
;
; PURPOSE:
;       Shrink or expand the size of an array by an arbitrary amount.
;       This IDL procedure simulates the action of the VAX/VMS
;       CONGRID/CONGRIDI function.
;
;	This function is similar to "REBIN" in that it can resize a
;       one, two, or three dimensional array.   "REBIN", however,
;       requires that the new array size must be an integer multiple
;       of the original size.   CONGRID will resize an array to any
;       arbitrary size (REBIN is somewhat faster, however).
;       REBIN averages multiple points when shrinking an array,
;       while CONGRID just resamples the array.
;
; CATEGORY:
;       Array Manipulation.
;
; CALLING SEQUENCE:
;	array = CONGRID(array, x, y, z)
;
; INPUTS:
;       array:  A 1, 2, or 3 dimensional array to resize.
;               Data Type : Any type except string or structure.
;
;       x:      The new X dimension of the resized array.
;               Data Type : Int or Long (greater than or equal to 2).
;
; OPTIONAL INPUTS:
;       y:      The new Y dimension of the resized array.   If the original
;               array has only 1 dimension then y is ignored.   If the
;               original array has 2 or 3 dimensions then y MUST be present.
;
;       z:      The new Z dimension of the resized array.   If the original
;               array has only 1 or 2 dimensions then z is ignored.   If the
;               original array has 3 dimensions then z MUST be present.
;
; KEYWORD PARAMETERS:
;       INTERP: If set, causes linear interpolation to be used.
;               Otherwise, the nearest-neighbor method is used.
;
;	CUBIC:	If specified and non-zero, "Cubic convolution"
;		interpolation is used.  This is a more
;		accurate, but more time-consuming, form of interpolation.
;		CUBIC has no effect when used with 3 dimensional arrays.
;		If this parameter is negative and non-zero, it specifies the
;		value of the cubic interpolation parameter as described
;		in the INTERPOLATE function.  Valid ranges are -1 <= Cubic < 0.
;		Positive non-zero values of CUBIC (e.g. specifying /CUBIC)
;		produce the default value of the interpolation parameter
;		which is -1.0.
;
;       MINUS_ONE:
;               If set, will prevent CONGRID from extrapolating one row or
;               column beyond the bounds of the input array.   For example,
;               If the input array has the dimensions (i, j) and the
;               output array has the dimensions (x, y), then by
;               default the array is resampled by a factor of (i/x)
;               in the X direction and (j/y) in the Y direction.
;               If MINUS_ONE is present (AND IS NON-ZERO) then the array
;               will be resampled by the factors (i-1)/(x-1) and (j-1)/(y-1).
;
; OUTPUTS:
;	The returned array has the same number of dimensions as the original
;       array and is of the same data type.   The returned array will have
;       the dimensions (x), (x, y), or (x, y, z) depending on how many
;       dimensions the input array had.
;
; PROCEDURE:
;       IF the input array has three dimensions, or if INTERP is set,
;       then the IDL interpolate function is used to interpolate the
;       data values.
;       If the input array has two dimensions, and INTERP is NOT set,
;       then the IDL POLY_2D function is used for nearest neighbor sampling.
;       If the input array has one dimension, and INTERP is NOT set,
;       then nearest neighbor sampling is used.
;
; EXAMPLE:
;       ; vol is a 3-D array with the dimensions (80, 100, 57)
;       ; Resize vol to be a (90, 90, 80) array
;       vol = CONGRID(vol, 90, 90, 80)
;
; MODIFICATION HISTORY:
;       DMS, Sept. 1988.
;       DMS, Added the MINUS_ONE keyword, Sept. 1992.
; 	Daniel Carr. Re-wrote to handle one and three dimensional arrays
;                    using INTERPOLATE function.
; 	DMS, RSI, Nov, 1993.  Added CUBIC keyword.
;-

FUNCTION CONGRID, arr, x, y, z, Interp=int, Minus_One=m1, Cubic = cubic

ON_ERROR, 2		;Return to caller if error
s = Size(arr)

IF ((s[0] EQ 0) OR (s[0] GT 3)) THEN $
   Message, 'Array must have 1, 2, or 3 dimensions.'

;  Supply defaults = no interpolate, and no minus_one.
if n_elements(int) le 0 then int = 0 else int = keyword_set(int)
if n_elements(m1) le 0 then m1 = 0 else m1 = keyword_set(m1)
if n_elements(cubic) eq 0 then cubic = 0
if cubic ne 0 THEN int = 1	;Cubic implies interpolate
	

CASE s[0] OF
   1: BEGIN				; *** ONE DIMENSIONAL ARRAY
	srx = float(s[1] - m1)/(x-m1) * findgen(x)  ;subscripts
      IF int THEN $
         RETURN, INTERPOLATE(arr, srx, CUBIC = cubic) ELSE $
         RETURN, arr[ROUND(srx)]
   ENDCASE
   2: BEGIN ; *** TWO DIMENSIONAL ARRAY
	IF int THEN BEGIN
	  srx = float(s[1] - m1) / (x-m1) * findgen(x)
	  sry = float(s[2] - m1) / (y-m1) * findgen(y)
          RETURN, INTERPOLATE(arr, srx, sry, /GRID, CUBIC=cubic)
	ENDIF ELSE $
	  RETURN, POLY_2D(arr, $
		[[0,0],[(s[1]-m1)/float(x-m1),0]], $ ;Use poly_2d
		[[0,(s[2]-m1)/float(y-m1)],[0,0]],int,x,y)

   ENDCASE
   3: BEGIN ; *** THREE DIMENSIONAL ARRAY
	srx = float(s[1] - m1) / (x-m1) * findgen(x)
	sry = float(s[2] - m1) / (y-m1) * findgen(y)
	srz = float(s[3] - m1) / (z-m1) * findgen(z)
	RETURN, interpolate(arr, srx, sry, srz, /grid)
   ENDCASE
ENDCASE

RETURN, arr_r
END


function minmax,array
;+
; NAME:
;   MINMAX
; PURPOSE:
;   Return a 2 element array giving the minimum and maximum of a vector
;   or array.  This is faster than doing a separate MAX and MIN.
; CALLING SEQUENCE:
;   value = minmax( array )
; INPUTS:
;   array - an IDL numeric scalar, vector or array.
; OUTPUTS:
;   value = a two element vector,
;           value(0) = minimum value of array
;           value(1) = maximum value of array
; EXAMPLE:
;   Print the minimum and maximum of an image array, im
;
;         IDL> print, minmax( im )
; PROCEDURE:
;   The MIN function is used with the MAX keyword
; REVISION HISTORY:
;   Written W. Landsman                January, 1990
;-
On_error,2
amin = min( array, MAX = amax)
return, [ amin, amax ]
end


;+
; NAME:
;       RESIZE
; PURPOSE:
;       Resize 1- or 2-dimensional arrays.
;       Much like the built-in function REBIN, but the restriction to 
;       integral multiples or fractions of the original size is dropped.
; CATEGORY:
;       Image processing.
; CALLING SEQUENCE:
;       a=resize(b,nx,ny)     ; for 2-dim arrays
;       a=resize(b,n)         ; for 1-dim arrays
; INPUTS:
;       b = data array to be resized
;       nx,ny = new dimensions
; KEYWORD PARAMETERS:
;       /sample : If set, nearest neighborsampling is used (like with REBIN).
; OUTPUTS:
;       function value = resized data array
; COMMON BLOCKS:
;       none
; SIDE EFFECTS:
;       none
; RESTRICTIONS:
;       Only for one or two dimensional arrays.
; PROCEDURE:
;       The built-in function POLY_2D is used for (bi-)linear interpolation.
;       If the new dimension is smaller than the old one, and /sample is
;       not set, the array is first enlarged to the smallest possible
;       integral multiple of the new dimension, and then averaged using REBIN.
; MODIFICATION HISTORY:
;    Author : A. Welz,  Uni. Wuerzburg, Germany , Feb. 1991
;-
function resize,dati,nxnew,nynew,sample=sample
;
on_error,2
;
if n_elements(nxnew) eq 0 then nxnew=1
if n_elements(nynew) eq 0 then nynew=1
nxnew=fix(nxnew)>1 & nynew=fix(nynew)>1
;
data=reform(dati)
s=size(data)
if s(0) eq 1 then begin
    data=reform(data,s(1),1)
    s=size(data)
endif
if s(0) ne 2 then goto,errex
;
type=1. < abs(data(0,0)) > 1.
nxold=s(1)
nyold=s(2)
;
nx=nxnew
ny=nynew
if not keyword_set(sample) then begin
   if nxnew lt nxold then nx=nxnew*long(nxold/nxnew+1)
   if nynew lt nyold then ny=nynew*long(nyold/nynew+1)
endif
;
   d=fltarr(nxold+1,nyold+1)*type
   d(0:nxold-1,0:nyold-1)=data
   data=d
   data(nxold,0:nyold-1)=data(nxold-1,0:nyold-1)
   data(*,nyold)=data(*,nyold-1)
;
xfac=float(nxold)/float(nx)
yfac=float(nyold)/float(ny)
p=fltarr(2,2) & q=p
p(0,1)=xfac
q(1,0)=yfac
;
if keyword_set(sample) then begin
   return,reform(poly_2d(data,p,q,0,nx,ny))
endif else begin
   return,reform(rebin(poly_2d(data,p,q,1,nx,ny),nxnew,nynew))
endelse
;
errex: print,' RESIZE: dimension must be 1 or 2'
return,dati
end
